# AUTOGENERATED! DO NOT EDIT! File to edit: driving.ipynb (unless otherwise specified).

__all__ = ['BG_COLOR', 'GRID_COLOR', 'read_ply', 'sys_byteorder', 'ply_dtypes', 'read_lidar_points', 'cloud_layout',
           'visualize_cloud', 'gen_color_palette', 'COLOR_OPTIONS', 'visualize_clouds', 'animation_layout',
           'visualize_clouds_animation']

# Cell
import sys
import numpy as np
import pandas as pd
from collections import defaultdict

import plotly.express as px
import plotly.graph_objects as go

BG_COLOR = 'rgb(30, 30, 30)'
GRID_COLOR = 'rgb(127, 127, 127)'


# Cell
# Adapted from pyntcloud under MIT license

sys_byteorder = ('>', '<')[sys.byteorder == 'little']

ply_dtypes = dict([
    (b'int8', 'i1'),
    (b'char', 'i1'),
    (b'uint8', 'u1'),
    (b'uchar', 'b1'),
    (b'uchar', 'u1'),
    (b'int16', 'i2'),
    (b'short', 'i2'),
    (b'uint16', 'u2'),
    (b'ushort', 'u2'),
    (b'int32', 'i4'),
    (b'int', 'i4'),
    (b'uint32', 'u4'),
    (b'uint', 'u4'),
    (b'float32', 'f4'),
    (b'float', 'f4'),
    (b'float64', 'f8'),
    (b'double', 'f8')
])


def read_ply(filename):
    """ Read a binary_little_endian .ply file and return data as a dict.

    Parameters:
        filename: of ply file

    Returns:
        A dictionary with `points`, `mesh`, and/or `comments` keys.
    """
    with open(filename, 'rb') as ply:

        if b'ply' not in ply.readline():
            raise ValueError('The file does not start with the word ply')

        # make sure format is binary_little_endian
        fmt = ply.readline().split()[1].decode()
        assert fmt == 'binary_little_endian'

        line = []
        dtypes = defaultdict(list)
        count = 2
        points_size = None
        mesh_size = None
        comments = []
        while b'end_header' not in line and line != b'':
            line = ply.readline()

            if b'element' in line:
                line = line.split()
                name = line[1].decode()
                size = int(line[2])
                if name == "vertex":
                    points_size = size
                elif name == "face":
                    mesh_size = size

            elif b'property' in line:
                line = line.split()
                # element mesh
                if b'list' in line:

                    if b"vertex_indices" in line[-1] or b"vertex_index" in line[-1]:
                        mesh_names = ["n_points", "v1", "v2", "v3"]
                    else:
                        mesh_names = ["n_coords"] + ["v1_u", "v1_v", "v2_u",
                                                     "v2_v", "v3_u", "v3_v"]

                    # the first number has different dtype than the list
                    dtypes[name].append(
                        (mesh_names[0], '<' + ply_dtypes[line[2]]))
                    # rest of the numbers have the same dtype
                    dt = '<' + ply_dtypes[line[3]]

                    for j in range(1, len(mesh_names)):
                        dtypes[name].append((mesh_names[j], dt))
                else:
                    dtypes[name].append(
                        (line[2].decode(), '<' + ply_dtypes[line[1]]))

            elif b'comment' in line:
                line = line.split(b" ", 1)
                comment = line[1].decode().rstrip()
                comments.append(comment)

            count += 1

        end_header = ply.tell()

    data = {}

    if comments:
        data["comments"] = comments

    with open(filename, 'rb') as ply:
        ply.seek(end_header)
        points_np = np.fromfile(ply, dtype=dtypes["vertex"], count=points_size)
        if sys_byteorder != '<':
            points_np = points_np.byteswap().newbyteorder()
        data["points"] = pd.DataFrame(points_np)
        if mesh_size:
            mesh_np = np.fromfile(ply, dtype=dtypes["face"], count=mesh_size)
            if sys_byteorder != '<':
                mesh_np = mesh_np.byteswap().newbyteorder()
            data["mesh"] = pd.DataFrame(mesh_np)
            data["mesh"].drop('n_points', axis=1, inplace=True)

    return data


# Cell
# adapted from code by 3630 TAs Binit Shah and Jerred Chen
def read_lidar_points(filename):
    """ Read 3D points in LIDAR scan stored as a binary_little_endian .ply file.

    Parameters:
        filename: of ply file

    Returns:
        A tuple (3,N) numpy array.
    """
    data = read_ply(filename)
    points = data["points"]
    np_cloud = np.empty((3, len(points)))
    np_cloud[0, :] = points['x']
    np_cloud[1, :] = points['y']
    np_cloud[2, :] = points['z']
    return np_cloud


# Cell
def cloud_layout(show_grid_lines):
    """Create layout for showing clouds."""
    grid_lines_color = GRID_COLOR if show_grid_lines else BG_COLOR
    layout = go.Layout(
        scene=dict(
            xaxis=dict(nticks=8,
                       showbackground=True,
                       backgroundcolor=BG_COLOR,
                       gridcolor=grid_lines_color,
                       zerolinecolor=grid_lines_color),
            yaxis=dict(nticks=8,
                       showbackground=True,
                       backgroundcolor=BG_COLOR,
                       gridcolor=grid_lines_color,
                       zerolinecolor=grid_lines_color),
            zaxis=dict(nticks=8,
                       showbackground=True,
                       backgroundcolor=BG_COLOR,
                       gridcolor=grid_lines_color,
                       zerolinecolor=grid_lines_color),
            xaxis_title="x (meters)",
            yaxis_title="y (meters)",
            zaxis_title="z (meters)"
        ),
        scene_aspectmode='data',
        margin=dict(r=10, l=10, b=10, t=10),
        paper_bgcolor=BG_COLOR,
        font=dict(
            family="Courier New, monospace",
            color=grid_lines_color
        ),
        legend=dict(
            font=dict(
                family="Courier New, monospace",
                color=GRID_COLOR
            )
        )
    )
    return layout


# Cell
def visualize_cloud(cloud, show_grid_lines=False,  color='#90FF90',
                    marker_size=1, fraction=None):
    """ Visualizes point cloud in 3D scatter plot.

    Args:
        cloud (np.ndarray):     point cloud, a (3, num_points) numpy array
        show_grid_lines (bool): plots gridlines
        color (str):            color for markers
        marker_size (int):      size of each marker
        fraction (double):      take only a fraction of the points
    """
    # Setup data
    data = []
    N = cloud.shape[1]
    if fraction is not None:
        subset = np.random.choice(N, int(N * fraction), replace=False)
    x_data = cloud[0][subset] if fraction is not None else cloud[0]
    y_data = cloud[1][subset] if fraction is not None else cloud[1]
    z_data = cloud[2][subset] if fraction is not None else cloud[2]

    data.append(go.Scatter3d(x=x_data, y=y_data, z=z_data,
                             mode='markers',
                             marker=dict(
                                 size=marker_size,
                                 color=color,
                                 opacity=1.0
                             ))
                )
    fig = go.Figure(data=data, layout=cloud_layout(show_grid_lines))
    fig.show()


# Cell
COLOR_OPTIONS = ['#FF1493', '#CD5C5C', '#FFDAB9', '#FF4500', '#8B0000', '#E6E6FA', '#87CEFA', '#CD853F', '#DC143C',
                 '#808000', '#483D8B', '#D2691E', '#FF69B4', '#8FBC8F', '#A0522D', '#9932CC', '#B22222', '#F08080',
                 '#FA8072', '#4B0082', '#EE82EE', '#008000', '#F0F8FF', '#00FA9A', '#FFA500', '#BA55D3', '#0000FF',
                 '#66CDAA', '#FFF8DC', '#9370DB', '#00BFFF', '#FFFFFF', '#FF0000', '#3CB371', '#4682B4', '#FFFAF0',
                 '#FFFFE0', '#FFD700', '#800000', '#A52A2A', '#7FFF00', '#DDA0DD', '#F5F5F5', '#EEE8AA', '#F5F5DC',
                 '#FFC0CB', '#6495ED', '#8A2BE2', '#C0C0C0', '#F5FFFA', '#FF8C00', '#20B2AA', '#48D1CC', '#E0FFFF',
                 '#87CEEB', '#4169E1', '#FF7F50', '#F8F8FF', '#DAA520', '#B0C4DE', '#F4A460', '#00CED1', '#2E8B57',
                 '#7CFC00', '#7FFFD4', '#FFB6C1', '#B8860B', '#8B4513', '#8B008B', '#BC8F8F', '#663399', '#C71585',
                 '#F0FFF0', '#D2B48C', '#F0E68C', '#00FF00', '#BDB76B', '#5F9EA0', '#ADD8E6', '#F0FFFF', '#1E90FF',
                 '#FFF5EE', '#FFA07A', '#778899', '#ADFF2F', '#DB7093', '#FFE4E1', '#FF00FF', '#32CD32', '#FFFF00',
                 '#DCDCDC', '#9ACD32', '#FFDEAD', '#DA70D6', '#008B8B', '#6A5ACD', '#008080', '#D8BFD8', '#00FF7F',
                 '#FF6347', '#228B22', '#6B8E23', '#708090', '#556B2F', '#40E0D0', '#98FB98', '#90EE90', '#7B68EE',
                 '#696969', '#E9967A', '#00FFFF', '#F5DEB3', '#FFFACD', '#D3D3D3', '#AFEEEE', '#FFF0F5', '#191970']


def gen_color_palette(n):
    """Generates a hex color palette of size n, without repeats
    and only light colors (easily visible on dark background).

    Args:
        n (int): number of clouds, each cloud gets a unique color
    """
    palette = []
    do_replace = False if len(COLOR_OPTIONS) >= n else True
    for i in np.random.choice(len(COLOR_OPTIONS), n, replace=do_replace):
        palette.append(COLOR_OPTIONS[i])

    return palette


# Cell
def visualize_clouds(clouds, show_grid_lines=False, cloud_colors=None,
                     marker_size=1, do_subsampling=True):
    """Visualizes cloud(s) in a iterative 3D plot.
    Due to browser limitations, rendering above 5 frames requires
    subsampling of the point clouds, which is done automatically.

    Example input of arg:
    clouds = [clouda, cloudb, cloudc]
    where each cloud is a numpy array of shape (3, num_points).
    cloud[0] are the x coordinates, cloud[1] is y, and cloud[2] is z.

    Args:
        clouds (list):          ordered series of point clouds
        show_grid_lines (bool): plots gridlines
        cloud_colors (list):    colors for each cloud in the visualization
        marker_size (int):      size of each marker
        do_subsampling (bool):  whether or not subsampling occurs
    """
    # Setup data
    nc = len(clouds)
    palette = gen_color_palette(nc)
    if cloud_colors is not None:
        if isinstance(cloud_colors, list):
            if nc != len(cloud_colors):
                raise ValueError(
                    'length of cloud_colors does not match length of clouds')
        else:
            cloud_colors = [cloud_colors] * nc
    data = []
    for i, cloud in enumerate(clouds):
        N = cloud.shape[1]
        subset = np.random.choice(N,
                                  int(N / (nc * 0.25)) if do_subsampling and int(
                                      N / (nc * 0.25)) <= N else N,
                                  replace=False)
        x_data = cloud[0][subset] if nc > 5 else cloud[0]
        y_data = cloud[1][subset] if nc > 5 else cloud[1]
        z_data = cloud[2][subset] if nc > 5 else cloud[2]

        data.append(go.Scatter3d(x=x_data, y=y_data, z=z_data,
                                 mode='markers',
                                 marker=dict(
                                     size=marker_size,
                                     color=palette[i] if not cloud_colors else cloud_colors[i],
                                     opacity=1.0
                                 ))
                    )

    fig = go.Figure(data=data, layout=cloud_layout(show_grid_lines))
    fig.show()


# Cell
def animation_layout(clouds_labels, speed):
    """Setup layout for animation."""
    return go.Layout(
        hovermode=False,
        sliders=[dict(
            active=0,
            yanchor="top",
            xanchor="left",
            currentvalue=dict(
                font=dict(
                    family="Courier New, monospace",
                    color=BG_COLOR
                ),
                prefix="Step ",
                visible=True,
                xanchor="right"
            ),
            pad=dict(b=10, t=0),
            len=0.95,
            x=0.05,
            y=0,
            font=dict(
                family="Courier New, monospace",
                color=GRID_COLOR
            ),
            steps=[dict(
                args=[[clouds_label], dict(
                    frame=dict(duration=speed),
                    mode="immediate"
                )],
                label=clouds_label,
                method="animate") for clouds_label in clouds_labels]
        )],
        updatemenus=[dict(
            buttons=[dict(
                args=[None, dict(
                    frame=dict(duration=speed),
                    fromcurrent=True
                )],
                label="Play",
                method="animate"
            )],
            direction="left",
            pad=dict(r=10, t=30),
            showactive=False,
            type="buttons",
            x=0.05,
            xanchor="right",
            y=0,
            yanchor="top",
            font=dict(
                family="Courier New, monospace",
                color=GRID_COLOR
            )
        )],
    )


# Cell
def visualize_clouds_animation(clouds_series, clouds_labels=None,
                               show_grid_lines=False, speed=100,
                               cloud_colors=None, marker_size=1,
                               do_subsampling=True):
    """Visualizes a cloud(s) series using a slider & play/pause.
    Due to browser limitations, rendering above 5 frames requires
    subsampling of the point clouds, which is done automatically.

    Example input of arg:
    clouds_series = [[clouda, cloudb], [clouda, cloudb]]
    clouds_labels = ["iteration1", "iteration2"]

    Args:
        clouds_series (list):   ordered series of point clouds
        clouds_labels (list):   ordered labels for clouds
        show_grid_lines (bool): plots gridlines
        speed (int):            speed at which the frames are played through
        cloud_colors (list):    colors for each cloud in the visualization
        marker_size (int):      size of each marker
        do_subsampling (bool):  whether or not subsampling occurs
    """
    clouds_labels = [str(i) for i in range(len(clouds_series))
                     ] if not clouds_labels else clouds_labels

    first_data = None
    palette = gen_color_palette(len(clouds_series[0]))
    if cloud_colors is not None:
        if isinstance(cloud_colors, list):
            if len(clouds_series[0]) != len(cloud_colors):
                raise ValueError(
                    'length of cloud_colors does not match length of clouds')
        else:
            raise ValueError('cloud_colors must be a list')
    frames = []
    for i, (label, clouds) in enumerate(zip(clouds_labels, clouds_series)):
        data = []
        for j, cloud in enumerate(clouds):
            nc = len(clouds)
            N = cloud.shape[1]
            subset = np.random.choice(N,
                                      int(N / (nc * 0.25)) if do_subsampling and int(
                                          N / (nc * 0.25)) <= N else N,
                                      replace=False)
            x_data = cloud[0][subset] if nc > 5 else cloud[0]
            y_data = cloud[1][subset] if nc > 5 else cloud[1]
            z_data = cloud[2][subset] if nc > 5 else cloud[2]

            data.append(go.Scatter3d(x=x_data, y=y_data, z=z_data,
                                     mode='markers',
                                     marker=dict(
                                         size=marker_size,
                                         color=palette[j] if not cloud_colors else cloud_colors[j],
                                         opacity=1.0
                                     ))
                        )
        frames.append(go.Frame(name=label, data=data))
        if first_data is None:
            first_data = data

    fig = go.Figure(data=first_data, layout=cloud_layout(
        show_grid_lines), frames=frames)
    fig.update_layout(animation_layout(clouds_labels, speed))
    fig.show()



# AUTOGENERATED! DO NOT EDIT! File to edit: cli.ipynb (unless otherwise specified).

__all__ = ['rename', 'fix_colab_links', 'REGEX', 'fix_nbs_colab_links']

# Cell
from pathlib import Path
from fastcore.script import call_parse
import json
import re
import io


# Cell
@call_parse
def rename(dir: str,  # dir in which to rename files
           suffix: str = ".ipynb"  # suffix of files to change
           ):
    """Rename notebooks to base 1 for chapters."""
    path = Path(dir)
    assert path.exists(), f"dir '{dir}' not found"
    assert path.is_dir(), f"'{dir}' is not a directory"
    print("Renaming notebooks now.")
    for ch in list(range(8, -1, -1)):
        print(f"Renaming chapter {ch}:")
        for x in path.glob(f"S{ch}*{suffix}"):
            new_name = path / x.name.replace(f"S{ch}", f"S{ch+1}")
            print(f"Renaming {x} to {new_name}")
            x.rename(new_name)


# Cell

REGEX = re.compile(
    '<a href="https://colab.research.google.com/github/gtbook/robotics/blob/main/(.+?)\.ipynb".*')


def _colab_link_name(nb):
    colab_link_source = nb['cells'][0]['source'][0]
    result = re.search(REGEX, colab_link_source)
    return result.group(1) if result is not None else None


def fix_colab_links(nb: dict, name: str):
    "Fix colab links in `nb`"

    # Fix colab link
    before = _colab_link_name(nb)
    if before is not None:
        colab_link_source = nb['cells'][0]['source'][0]
        nb['cells'][0]['source'][0] = colab_link_source.replace(before, name)

    # Fix colab metadata
    if 'colab' in nb['metadata']:
        nb['metadata']['colab']['name'] = name + '.ipynb'


# Cell
@call_parse
def fix_nbs_colab_links(dir: str,  # dir in which to rename files
                        ):
    """Fix colab links in all notebooks in directory."""
    path = Path(dir)
    assert path.exists(), f"dir '{dir}' not found"
    assert path.is_dir(), f"'{dir}' is not a directory"
    print("Fixing notebooks now.")
    for x in path.glob("*.ipynb"):
        nb = json.loads(open(str(x), 'r', encoding='utf-8').read())
        fix_colab_links(nb, str(x.stem))
        all = json.dumps(nb, sort_keys=True, indent=1, ensure_ascii=False)
        with io.open(x, 'w', encoding='utf-8') as f:
            f.write(all)
            f.write("\n")



# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"rename": "cli.ipynb",
         "fix_colab_links": "cli.ipynb",
         "REGEX": "cli.ipynb",
         "fix_nbs_colab_links": "cli.ipynb",
         "dbn_writer": "dbn.ipynb",
         "has_positions": "dbn.ipynb",
         "Axis": "dbn.ipynb",
         "axes": "diffdrive.ipynb",
         "plane": "diffdrive.ipynb",
         "ray": "diffdrive.ipynb",
         "show_3d": "diffdrive.ipynb",
         "read_image": "diffdrive.ipynb",
         "conv2": "diffdrive.ipynb",
         "Variables": "discrete.ipynb",
         "DiscreteKey": "discrete.ipynb",
         "show": "display.ipynb",
         "pretty": "display.ipynb",
         "randomImages": "display.ipynb",
         "ROBOTS": "display.ipynb",
         "BG_COLOR": "driving.ipynb",
         "GRID_COLOR": "driving.ipynb",
         "read_ply": "driving.ipynb",
         "sys_byteorder": "driving.ipynb",
         "ply_dtypes": "driving.ipynb",
         "read_lidar_points": "driving.ipynb",
         "cloud_layout": "driving.ipynb",
         "visualize_cloud": "driving.ipynb",
         "gen_color_palette": "driving.ipynb",
         "COLOR_OPTIONS": "driving.ipynb",
         "visualize_clouds": "driving.ipynb",
         "animation_layout": "driving.ipynb",
         "visualize_clouds_animation": "driving.ipynb",
         "td": "html.ipynb",
         "tr": "html.ipynb",
         "th": "html.ipynb",
         "table": "html.ipynb",
         "row": "html.ipynb",
         "ROW": "html.ipynb",
         "vv": "linear.ipynb",
         "gaussian": "logistics.ipynb",
         "map_coords": "logistics.ipynb",
         "show_map": "logistics.ipynb",
         "shelf_width": "logistics.ipynb",
         "shelf_gap": "logistics.ipynb",
         "spacing": "logistics.ipynb",
         "side": "logistics.ipynb",
         "bottom": "logistics.ipynb",
         "top": "logistics.ipynb",
         "base_map": "logistics.ipynb",
         "beacons": "logistics.ipynb",
         "proximity_range": "logistics.ipynb",
         "proximity_map_on": "logistics.ipynb",
         "proximity_map_off": "logistics.ipynb",
         "rfid_range": "logistics.ipynb",
         "rfid_measurement": "logistics.ipynb",
         "out_of_rfid_range": "logistics.ipynb",
         "out_of_bound_map": "logistics.ipynb"}

modules = ["cli.py",
           "dbn.py",
           "diffdrive.py",
           "discrete.py",
           "display.py",
           "driving.py",
           "html.py",
           "linear.py",
           "logistics.py"]

doc_url = "https://gtbook.github.io//gtbook/"

git_url = "https://github.com/gtbook/gtbook/tree/master/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: logistics.ipynb (unless otherwise specified).

__all__ = ['gaussian', 'map_coords', 'show_map', 'base_map', 'beacons', 'proximity_map_on', 'proximity_map_off',
           'rfid_range', 'rfid_measurement', 'out_of_rfid_range', 'out_of_bound_map']

# Cell
import gtsam
import math
import numpy as np

# Cell
def gaussian(x: np.array, mean=np.zeros((2,)), cov=np.eye(2)):
    """Evaluate multivariate Gaussian at x of shape(m,n), yields (m,) vector."""
    assert x.shape[-1] == 2, f"error: x has shape {x.shape}"
    k = math.sqrt(np.linalg.det(2*math.pi*cov))
    e = x - mean
    E = np.sum(e * (e @ np.linalg.inv(cov)), axis=-1)
    return np.exp(-E)/k


# Cell
X, Y = np.meshgrid(np.linspace(0, 99, 100), np.linspace(0, 49, 50))
map_coords = np.dstack((X, Y))


# Cell
import plotly.express as px


def show_map(image=None, markers=None, file: str = None, marker={}):
    """Show image on warehouse map, possibly with markers"""
    fig = px.imshow(image, color_continuous_scale='Oranges',
                    origin="lower", aspect='equal', range_color=[0, 1])
    if markers is not None:
        fig.add_scatter(x=markers[:, 0], y=markers[:, 1],
                        mode="markers", marker=marker)
    fig.update_layout(coloraxis_showscale=False,
                      margin=dict(l=0, r=0, t=0, b=0))
    fig.update_xaxes(range=[-0.5, 99.5], autorange=False)
    fig.update_yaxes(range=[-0.5, 49.5], autorange=False,scaleratio = 1)
    fig.show()
    if file is not None:
        fig.write_image(file)


# Internal Cell

shelf_width = 9
shelf_gap = 8

spacing = shelf_width + shelf_gap
side = (100 - shelf_width*4 - shelf_gap*3) // 2

bottom = [[float(shelf_width)/2+side+i*spacing-0.5, 9.5] for i in range(4)]
top = [[float(shelf_width)/2+side+i*spacing-0.5, 39.5] for i in range(4)]


# Cell

base_map = np.zeros((50, 100))
for i in range(4):
    base_map[10:40, side+i*spacing:side+shelf_width+i*spacing] = 1

beacons = np.array(bottom + top)


# Internal Cell
proximity_range = 2  # what should be the proper proximity range ?


# Cell
proximity_map_on = np.zeros((50, 100))
for i in range(4):
    proximity_map_on[10-proximity_range:40+proximity_range, side+i *
                     spacing-proximity_range:side+shelf_width+i*spacing+proximity_range] = 1
proximity_map_on[0:proximity_range] = 1
proximity_map_on[-proximity_range:] = 1
proximity_map_on[:, -proximity_range:] = 1
proximity_map_on[:, 0:proximity_range:] = 1

for i in range(4):
    proximity_map_on[10:40, side+i*spacing:side+shelf_width+i*spacing] = 0


# Cell
proximity_map_off = np.full((50, 100), 1)
for i in range(4):
    proximity_map_off[10-proximity_range:40+proximity_range, side+i *
                      spacing-proximity_range:side+shelf_width+i*spacing+proximity_range] = 0

for i in range(4):
    proximity_map_off[10:40, side+i*spacing:side+shelf_width+i*spacing] = 0

proximity_map_off[0:proximity_range] = 0
proximity_map_off[-proximity_range:] = 0
proximity_map_off[:, -proximity_range:] = 0
proximity_map_off[:, 0:proximity_range:] = 0


# Cell

def rfid_range(position, beacon, max_range=8):
    """Return range to given beacon."""
    range = np.linalg.norm(position-beacon)
    return float('inf') if range > max_range else range


def rfid_measurement(position, max_range=8):
    """Simulate RFID reader that returns nearest RFID range or (None,inf)."""
    ranges = [rfid_range(position, beacon, max_range) for beacon in beacons]
    range = min(ranges)
    return (np.argmin(ranges), range) if range <= max_range else (None, range)


def out_of_rfid_range(position, max_range=8):
    """Check if all beacons are out of range."""
    id, _ = rfid_measurement(position, max_range)
    return id == None

out_of_bound_map = np.array([[out_of_rfid_range(xy)
                            for xy in row] for row in map_coords])


# AUTOGENERATED! DO NOT EDIT! File to edit: linear.ipynb (unless otherwise specified).

__all__ = ['vv']

# Cell
import gtsam

from typing import List, Tuple, Callable, Dict, Iterable


# Cell
import gtsam
import numpy as np

# Cell
def vv(keys_vectors):
    """Create a VectorValues from a dict"""
    result = gtsam.VectorValues()
    for j, v in keys_vectors.items():
        result.insert(j, v)
    return result